package com.zehong.netty.decode;

import cn.hutool.core.date.DateUtil;
import com.zehong.config.NettyClientConfig;
import com.zehong.utils.HexUtils;
import com.zehong.utils.NettyClient;
import com.zehong.utils.RedisUtil;
import com.zehong.utils.SpringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.zip.CRC32;

/**
 * @author gengd
 * 信息单元分包信息解析
 */
public class PackageInformationDecode {

    private Logger logger = LoggerFactory.getLogger(PackageInformationDecode.class);

    /**
     * 上传信息解析数据
     */
    private MsgDecode decode;

    /**
     * 分包信息
     */
    private String packageInformation;

    /**
     * 总包数
     */
    private String totalPackageCount;

    /**
     * 当前包数
     */
    private String currentPackageIndex;

    /**
     * 当前包是否发送过
     */
    public boolean isNotSaveCurrentIndex;

    public PackageInformationDecode(MsgDecode decode){
        this.decode = decode;
        this.packageInformation = decode.getMsgInfo().substring(28,42);
    }

    public void decodePakageInformation(){
        //信息单元类型
        String ieType = this.packageInformation.substring(0,2);
        //长度
        String length = this.packageInformation.substring(2,6);
        //总包数
        this.totalPackageCount = this.packageInformation.substring(6,10);
        //当前包数
        this.currentPackageIndex = this.packageInformation.substring(10,14);
        logger.info("transmissionId:" + decode.getTransmissionId() + "分包信息： 信息单元类型："+ ieType +" 长度：" + length + "总包数：" + totalPackageCount + "当前包数" + currentPackageIndex);
        //缓存分包信息
        packageInformationCash();
    }

    /**
     * 缓存分包信息
     */
    private void packageInformationCash(){
       RedisUtil redisUtil = SpringUtil.getBean(RedisUtil.class);
       List<String> cashInfo = (List<String>) redisUtil.get(decode.getTransmissionId());
       //缓存信息不为空
       if(null != cashInfo && !cashInfo.isEmpty()){
           if(cashInfo.contains(currentPackageIndex)){
              return;
           }
           cashInfo.add(currentPackageIndex);
           isNotSaveCurrentIndex = true;
           //如接受到所有分包信息，移除分包缓存
           if( Integer.parseInt(totalPackageCount,16) == cashInfo.size()){
               //确认消息
               confirmInformation();
               //移除分包缓存
               redisUtil.del(decode.getTransmissionId());
               return;
           }
           // 缓存分包信息
           redisUtil.set(decode.getTransmissionId(),cashInfo,60 * 60 *6);
           return;
       }
       isNotSaveCurrentIndex = true;
       List<String> packageInfoCash = new ArrayList<>();
       packageInfoCash.add(currentPackageIndex);
       redisUtil.set(decode.getTransmissionId(),packageInfoCash,60 * 60 *6);
    }

    /**
     * 确认成功接收消息
     */
    public void confirmInformation(){
        String head = decode.getMsgHead();
        String tail = decode.getMsgTail();
        String dateTime = getDateTimeHex();
        CRC32 crc32 = new CRC32();
        crc32.update((head.substring(8,head.length())+dateTime).getBytes());
        String checkCode = Long.toHexString(crc32.getValue());
        StringBuilder reqMsg = new StringBuilder(head);
        reqMsg.append(dateTime)
              .append(checkCode)
              .append(tail.substring(8,12));
        NettyClientConfig config = SpringUtil.getBean(NettyClientConfig.class);
        String msg = NettyClient.nettyConnection(reqMsg.toString(),config.getHost(),config.getPort());
        logger.info("transmissionId:" + decode.getTransmissionId() + "消息确认服务器回复消息：" + msg);
    }

    /**
     * 拼接信息单元
     * @return String
     */
    private static String getDateTimeHex(){
        String now = DateUtil.now();
        String[] date =now.split(" ");
        //年月日
        String[] yearDate = date[0].split("-");
        //时分秒
        String[] time = date[1].split(":");
        System.out.println("年：" + HexUtils.bytesToHex(yearDate[0].substring(0,2).getBytes()) + " 时：" + time.toString());
        StringBuilder dateTime = new StringBuilder("");
        dateTime.append(String.format("%02x",Integer.parseInt(yearDate[0].substring(0,2))))
                .append(String.format("%02x",Integer.parseInt(yearDate[0].substring(2,4))))
                .append(String.format("%02x",Integer.parseInt(yearDate[1])))
                .append(String.format("%02x",Integer.parseInt(yearDate[2])))
                .append(String.format("%02x",Integer.parseInt(time[0])))
                .append(String.format("%02x",Integer.parseInt(time[1])))
                .append(String.format("%02x",Integer.parseInt(time[2])));
        return dateTime.toString();
    }
}
