package com.zehong.netty.handler;

import com.zehong.enums.MessageTypeEnum;
import com.zehong.netty.decode.InformationDecode;
import com.zehong.netty.decode.MsgDecode;
import com.zehong.utils.HexUtils;
import com.zehong.utils.RedisUtil;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.socket.SocketChannel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.UnsupportedEncodingException;
import java.util.Date;

/**
 * @author gengd
 * 物联网 开启检测端口 并写入数据库
 */
@Component
@Sharable
public class TcpServerHandler extends ChannelInboundHandlerAdapter {

    private Logger logger = LoggerFactory.getLogger(TcpServerHandler.class);

    @Autowired
    private RedisUtil redisUtil;

    @Override
    public void channelActive(ChannelHandlerContext ctx) {
        SocketChannel channel = (SocketChannel) ctx.channel();
        logger.info("链接报告开始");
        logger.info("链接报告信息：有一客户端链接到本服务端");
        logger.info("链接报告IP:{}", channel.localAddress().getHostString());
        logger.info("链接报告Port:{}", channel.localAddress().getPort());
        logger.info("链接报告完毕");
        //通知客户端链接建立成功
        String str = "通知客户端链接建立成功" + " " + new Date() + " " + channel.localAddress().getHostString() + "\r\n";
        ctx.writeAndFlush(str);
    }

    /**
     * 当客户端主动断开服务端的链接后，这个通道就是不活跃的。也就是说客户端与服务端的关闭了通信通道并且不可以传输数据
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx){
        logger.info("客户端断开链接{}", ctx.channel().localAddress().toString());
    }
    /**
     * 获取数据
     * @param ctx 上下文
     * @param msg 获取的数据
     * @throws UnsupportedEncodingException
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg){
        ByteBuf buf = (ByteBuf)msg;
        byte [] msgBytes = new byte[buf.readableBytes()];
        buf.readBytes(msgBytes);
        //16进制消息字符串
        String msgStr = HexUtils.bytesToHex(msgBytes);
        logger.info("tcp上传数据为：" + msgStr);
        //解析上传数据
        MsgDecode decode = new MsgDecode(msgStr);
        if(!MessageTypeEnum.READ_DATA_RESPONSE.value.equals(decode.getMessageType())){
            logger.info("此消息类型不做处理！");
            return;
        }
        if(!decode.checkCRC32Code()){
            logger.error("上传数据校验失败！");
            return;
        }
        //解析信息单元
        new InformationDecode(decode).infoMsgDecode();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        logger.error("tcp连接报错",cause);
        cause.printStackTrace();
        ctx.close();
    }
}
